// FCF ESTIMATOR 		mriedl v1.0.4 10dec2010 - enhanced mata version
// FIRST ARGUMENT: ordered dependent variable
// SECOND ARGUMENT: independent variable(s)


program fcf, eclass sortpreserve
	version 10


// SPECIFY SYTNAX AND MINIMUM OF VARIABLES
	syntax varlist(min=2) [if] [in] ,GRoup(varname) ///
	[, OR Level(integer 95) ITerate(integer 30) DEPvar(string) GROUPMean VCE(passthru) STARTvalues(numlist)]
	marksample touse
	gettoken left varlist: varlist, parse(": ")


		
// CHECK REQUIRED GROUP OPTION
	if `"`group'"' != "" {
		local panelid = `"`group'"'
	}
	else {
		di as err "group() required"
			exit 198
	}


// ASSGINING NAMES TO TEMPORARY VARIABLES AND SCALARS/MATRICES
	tempvar  T maxtrace min
	tempname B X TT sum cma b_1 V_1 
	

// MAXIMUM NUMBER OF OBSERVATIONS PER GROUP
	sort `panelid'
	by `panelid': egen `T' = count(`left') if `touse'

	

// RECODE CATEGORICAL VARIABLE TO STANDARD ORDERED VARIABLE
	tempvar y_cat
	egen `y_cat' = group(`left')


// OBTAIN GROUP MINIMA
	quietly sum `y_cat' if `touse'
	local sdif = r(max) - r(min)
	by `panelid': egen `min' = min(`y_cat') if `touse' 
	
	di ""
	di ""
	di "Mark observations for usage in regressions ..."

	
// MARK OBSERVATIONS TO USE IN ESTIMATION - OBS EXCEEDING GROUP MINIMA

	forvalues i =  1/`sdif' {
		tempvar sum`i' keep`i' y`i' min`i'
		gen `keep`i'' = 1
		replace `keep`i'' = 0 if `T' == 1
		replace `keep`i'' = 0 if `T' == .
		replace `keep`i'' = 0 if `y_cat'== .
		quietly gen `min`i'' = `min' + `i' - 1
		quietly gen `y`i'' = `y_cat'> `min`i'' if `keep`i'' == 1
		by `panelid': egen `sum`i'' = sum(`y`i'')
		replace `keep`i'' = 0 if `sum`i'' == `T'
		replace `keep`i'' = 0 if `sum`i'' == 0
	}


// CHECK OPTION FOR SETTING NUMBER OF ITERATIONS
	local itnum = `iterate'


// CHECK OPTION FOR USER-SPECIFIED STARTING VALUES
	if  `"`startvalues'"' != "" {
		matrix input `B' = (`startvalues')
		local indep `varlist'
		matrix list `B'
	}
	else {		
		di ""
		di "First clogit estimation..."
		di ""	

	// CHECK OPTION FOR INDIVIDUAL GROUP MEAN RECODING
		tempvar y_first y_cat_mean
		gen `y_first' = 0
		if `"`groupmean'"' != "" {
			by `panelid': egen `y_cat_mean' = mean(`y_cat')
			replace `y_first' = 1 if `y_cat' > `y_cat_mean'	
		}
		else {
			replace `y_first' = `y1'
		}

	// FIRST CONDITIONAL LOGIT ESTIMATION, GROUPED BY PANEL WITH MAXIMUM OF 30 ITERATIONS
		clogit `y_first' `varlist', group(`panelid') iterate(`itnum')	

	
	// FORWARD SELECTED FIRST-STEP ERETURN CONTENTS
		local N_1 = e(N)	
		local ll_1 = e(ll)	
		matrix `b_1' = e(b)
		matrix `V_1' = e(V)
		
		di "done!"
		di ""
		
	// CREATE ERROR DUMMY FOR NON-CONVERGENCE	
		if e(converged) == 0 {
			display "First-step estimation NOT CONVERGED"
			exit
		}	
		
	// OBTAIN MATRIX B FOR CALCULATION OF HESSIANS
		matrix `B' = e(b)
		local indep : colnames `B'

	}


// CALCULATE HESSIANS FOR EACH GROUP
	
	di ""
	di "Calculating individual hessians..."
	di ""

	forvalues i = 1/`sdif' {
		di "step `i' of `sdif'..."
		tempvar counter`i' t`i' max`i'
		quietly gen `t`i'' = .
		quietly egen `counter`i'' = group(`panelid') if `keep`i'' == 1
		egen `max`i'' = max(`counter`i'')
		local max = `max`i''
		forvalues j = 1/`max' {
			mkmat `indep' if `counter`i'' == `j', matrix(`X')
			mkmat `T' if `counter`i'' == `j', matrix(`TT')
			mkmat `sum`i'' if `counter`i'' == `j', matrix(`sum')
			local TTT = `TT'[1,1]
			local sumy = `sum'[1,1]
			hessi `X' `B' `TTT' `sumy'
			quietly replace `t`i'' = r(trace) if `counter`i'' == `j'
			matrix drop `X' `TT' `sum'
		}
	}

	di "done!"
	di ""

// OBTAIN TRACES OF HESSIANS AND SELECT DATA FOR SECOND STEP ACCORDINGLY
	
	di ""
	di "Adjusting final dependent variable... " 
	
	quietly gen `maxtrace' = .	

	forvalues i = 1/`sdif' {
		quietly replace `maxtrace' = `t`i'' if `maxtrace' == .
		quietly replace `maxtrace' = `t`i'' if `t`i'' < `maxtrace' & `t`i'' != .
	}

	forvalues i = 1/`sdif' {
		quietly replace `maxtrace' = `i' if `maxtrace' == `t`i''
	}

	tempvar yfcf
	quietly gen `yfcf' = .

	forvalues i = 1/`sdif' {
		replace `yfcf' = `y`i'' if `maxtrace' == `i'
	}


// CHECK FOR OPTIONS TO BE APPLIED IN SECOND-STEP ESTIMATION

	if `"`vce'"' != "" {		// OPTIONS FOR STANDARD ERRORS	
		local vcetype = `"`vce'"'
	}
	else {
		local vcetype = ""
	}
	
	local cilevel = `level'		// USE USER-SPECIFIED CONFIDENCE INTERVALS

	if `"`or'"' != "" {		// OPTIONALLY DISPLAY ESTIMATES AS ODDS RATIOS
		local odds = `"`or'"'
	}
	else {
		local odds = ""
	}
	
	
// SECOND-STEP CONDITIONAL LOGIT ESTIMATION																				
	di ""
	di "Second-step clogit Estimation..."
	di ""
	
	clogit `yfcf' `varlist', group(`panelid') level(`cilevel') `odds' iterate(`itnum') `vcetype'
	

// OPTION FOR EXPLICITELY ADDING THE SECOND-STEP DEPVAR TO DATASET
	if `"`depvar'"' != "" {
		gen `depvar' = `yfcf'
	}
	

// CHECK FOR CONVERGENCE
	local error2 = 0	

	if e(converged) == 0 {
		display "Second-step estimation NOT CONVERGED"
		local error2 = 1
	}	


// GENERATE ADDITIONAL ERETURN CONTENTS
	capture ereturn scalar N_1 = `N_1'
	capture ereturn scalar er_2 = `error2'
	capture ereturn scalar ll_1 = `ll_1'
	capture ereturn matrix b_1 = `b_1'
	capture ereturn matrix V_1 = `V_1'

end

// CALCULATE COMBINATION MATRIX 'cmatrix'
// FIRST ARGUMENT: numy
// SECOND ARGUMENT: num1
// RETURN: Mata Matrix z
// NOTE: Mata Matrix z must be defined before 

capture mata: mata drop cma()
	mata:
	void cma(real scalar numy, real scalar num1, matrix z)
	{
		
		if (num1 == 1) {
			cm = I(numy)
		}
		else if (num1==numy) {
			cm = J(1,num1,1)
		}
		else {
			x = 0
			y = 0
			cma(numy-1, num1,x)
			hcm1 = x
			cma(numy-1, num1-1,y)
			hcm2 = y
			cm = ((J(rows(hcm2),1,1),hcm2) \ (J(rows(hcm1),1,0),hcm1))
		}
		z = cm
	}
	end



// CALCULATE HESSIAN MATRIX OF FIRST CLOGIT ML-FUNCTION
// FIRST ARGUMENT : X-Matrix 
// SEDCOND ARGUMENT: Beta vector b
// THIRD ARGUMENT: Number of observations per Group
// FOURTH ARGUMENT: SUM of Y
// RETURN: TRACE OF HESSIAN MATRIX r(trace)

program hessi, rclass
	version 10
	tempname u x b S c J A o H t
	mata:	`x' = st_matrix("`1'")
	mata:	`b' = st_matrix("`2'")
	mata:	`S' = 0
	mata:	cma(`3', `4', `S')
	mata:	`c' = rows(`S')
	mata:	`J' = J(`c',1,1)
	mata:	`A' = exp(`S' * `x' * `b'')
	mata: `o' = `J''`A'
	mata: `H' = ((`S' * `x')'`A') * ((`S' * `x')'`A')' * (1 / `o')^2 - ((`S' * `x')' * diag(`A') * (`S' * `x')) * (1 / `o')
	mata: `t' = trace(`H')
	mata: st_numscalar("`u'",`t')
	mata: mata drop `S' `x' `b' `c' `J' `A' `o' `H' `t'
	return scalar trace = `u'
end
